module ps2_rcv(ps2_clk, ps2_data, clock50, reset, read_ack, write_ack, data_ready, data);
input ps2_clk;
input ps2_data;
input clock50; // 50 Mhz system clock
input reset;
input read_ack, write_ack;
output data_ready;
output [7:0] data;
reg ready_set;
reg [7:0] data;
reg data_ready;
reg read_char;
reg clock; // 25 Mhz internal clock

reg [3:0] incnt;
reg [8:0] shiftin;

reg [7:0] filter;
reg ps2_clk_filtered;

// data_ready is set to 1 when data is available.
// user should set read_ack to 1 and then to 0 to clear data_ready

always @ (posedge ready_set or posedge read_ack)
if (read_ack) data_ready <= 1'b0;
else data_ready <= 1'b1;

// divide-by-two 50MHz to 25MHz
always @(posedge clock50)
	clock <= ~clock;



// This process filters the raw clock signal coming from the ps2 
// using an eight-bit shift register and two AND gates

always @(posedge clock)
begin
   filter <= {ps2_clk, filter[7:1]};
   if (filter==8'b1111_1111) ps2_clk_filtered <= 1;
   else if (filter==8'b0000_0000) ps2_clk_filtered <= 0;
end


// This process reads in serial data coming from the terminal

always @(posedge ps2_clk_filtered)
begin
   if (reset==1)
   begin
      incnt <= 4'b0000;
      read_char <= 0;
   end
   else if (!ps2_data && (!read_char || write_ack)) 
   // this is either start bit or write acknowledgement bit
   begin
	if (!write_ack) read_char <= 1;
	ready_set <= 0;
   end
   else
   begin
	   // shift in next 8 data bits to assemble a data code	
	   if (read_char == 1)
   		begin
      		if (incnt < 9) 
      		begin
				incnt <= incnt + 1'b1;
				shiftin = { ps2_data, shiftin[8:1]};
				ready_set <= 0;
			end
		else
			begin
				incnt <= 0;
				data <= shiftin[7:0];
				read_char <= 0;
				ready_set <= 1;
			end
		end
	end
end

endmodule
